;===========================================================================================
; Button.asm  Fasm Flat assembler
; BUTTON class
; pas de resource, id, controle total de control class
; Programm par AsmGges France
;===========================================================================================

format PE GUI 4.0
entry start

; Inclusion des macros/quates/structures ncessaires au programme
; ----------------------------------------------------------------
  include 'win32a.inc'
  include 'macro\if.inc'

  ID_GBOX            = 100
  ID_PUSHBUTTON      = 101
  ID_CHECKBOX        = 102
  ID_AUTOCHECKBOX    = 103
  ID_RADIOBUTTON     = 104
  ID_AUTORADIOBUTTON = 105
  ID_3STATE          = 106
  ID_AUTO3STATE      = 107
  ID_GROUPBOX        = 108

;===========================================================================================
; Dbut du code programme
; -----------------------
section '.code' code readable executable

 start:
    ; Obtenir le handle du programme
    ; ------------------------------
    invoke GetModuleHandle,0
    mov [wc.hInstance],eax

    ; Chargement d'une icne spcifique
    ; ---------------------------------
    invoke LoadIcon,[wc.hInstance],IDI_FRANCE
    mov [wc.hIcon],eax

    ; Chargement d'un curseur systme
    ; -------------------------------
    invoke LoadCursor,0,IDC_ARROW
    mov [wc.hCursor],eax

    ; Valider la structure WNDCLASS
    ; -----------------------------
    invoke RegisterClass,wc

    ; Cration de la fenetre de l'application
    ; ---------------------------------------
    invoke CreateWindowEx,0,ClassName,Titre,WS_CAPTION+WS_POPUP+WS_SYSMENU+\
                          WS_MINIMIZEBOX,[WinX],[WinY],[WinW],[WinH],0,0,[wc.hInstance],0
    mov [hWnd],eax

    ; Affiche et rafraichit la fenetre
    ; --------------------------------
    invoke ShowWindow,[hWnd],SW_SHOWDEFAULT
    invoke UpdateWindow,[hWnd]

 ; Boucle des messages
 ; -------------------
 MessageLoop:
    invoke GetMessage,msg,NULL,0,0
    or eax,eax
    jz ExitProgram
    invoke TranslateMessage,msg
    invoke DispatchMessage,msg
    jmp MessageLoop

 ExitProgram:
    ; Fermeture de l'application
    ; --------------------------
    invoke ExitProcess,[msg.wParam]

;===========================================================================================
; Procdure principale Windows
;-------------------------------------------------------------------------------------------
proc WindowProc uses ebx esi edi, @hWnd, @uMsg, @wParam, @lParam

    .if [@uMsg] = WM_CLOSE
        ; Boite  message fin d'application
        ; ---------------------------------
        invoke MessageBox,[@hWnd],MsgEnd,ClassName,MB_OK+MB_ICONINFORMATION

        ; Demande fin d'application
        ; -------------------------
        invoke PostQuitMessage,0

    .elseif [@uMsg] = WM_CREATE

        ; Dfinir une fonte de texte
        ; --------------------------
        invoke CreateFont,8,0,0,0,FW_NORMAL,0,0,0,ANSI_CHARSET,OUT_DEFAULT_PRECIS,\
                       CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH+FF_SWISS,\
                                                                      MSSansSerif
        mov [hFontMSSS],eax                      ;font handle

        ; Construit tous les controles
        ; ----------------------------
        stdcall CreateControls,[@hWnd]

    .elseif [@uMsg] = WM_COMMAND

        ; Click souris pour identification du bouton actionn
        ; ---------------------------------------------------
        .if [@wParam] = BN_CLICKED shl 16+ID_PUSHBUTTON
            invoke MessageBox,[@hWnd],PushButton,ClassName,MB_OK+MB_ICONINFORMATION

        .elseif [@wParam] = BN_CLICKED shl 16+ID_CHECKBOX
            invoke MessageBox,[@hWnd],CheckBox,ClassName,MB_OK+MB_ICONINFORMATION

        .elseif [@wParam] = BN_CLICKED shl 16+ID_AUTOCHECKBOX
            invoke MessageBox,[@hWnd],AutoCheckBox,ClassName,MB_OK+MB_ICONINFORMATION

        .elseif [@wParam] = BN_CLICKED shl 16+ID_RADIOBUTTON
            invoke MessageBox,[@hWnd],RadioButton,ClassName,MB_OK+MB_ICONINFORMATION

        .elseif [@wParam] = BN_CLICKED shl 16+ID_AUTORADIOBUTTON
            invoke MessageBox,[@hWnd],AutoRadioButton,ClassName,MB_OK+MB_ICONINFORMATION

        .elseif [@wParam] = BN_CLICKED shl 16+ID_3STATE
            invoke MessageBox,[@hWnd],ThreeState,ClassName,MB_OK+MB_ICONINFORMATION

        .elseif [@wParam] = BN_CLICKED shl 16+ID_AUTO3STATE
            invoke MessageBox,[@hWnd],Auto3State,ClassName,MB_OK+MB_ICONINFORMATION

        .endif

    .else
        ; Traitement par dfaut des autres messages Windows
        ; -------------------------------------------------
        invoke DefWindowProc,[@hWnd],[@uMsg],[@wParam],[@lParam]
        ret

    .endif

    mov eax,0
    ret
endp

;===========================================================================================
proc CreateControls @hWnd

    ; Cration de la fenetre-enfant GroupBox
    ; --------------------------------------
    stdcall Dlg,Button,GbGges,[@hWnd],4,2,286,20,ID_GBOX,BS_GROUPBOX,0

    ;=======================================================================================
    ; Cration de la sous-fenetre PushButton
    ; --------------------------------------
    stdcall Dlg,Button,PushButton,[@hWnd],70,26,160,22,ID_PUSHBUTTON,BS_PUSHBUTTON,0h

    ; Cration de la sous-fenetre CheckBox
    ; ------------------------------------
    stdcall Dlg,Button,CheckBox,[@hWnd],70,50,100,24,ID_CHECKBOX,BS_CHECKBOX,0h

    ; Cration de la sous-fenetre AutoCheckBox
    ; ----------------------------------------
    stdcall Dlg,Button,AutoCheckBox,[@hWnd],70,76,120,24,ID_AUTOCHECKBOX,BS_AUTOCHECKBOX,0h

    ; Cration de la sous-fenetre RadioButton
    ; ---------------------------------------
    stdcall Dlg,Button,RadioButton,[@hWnd],70,102,110,24,ID_RADIOBUTTON,BS_RADIOBUTTON,0h

    ; Cration de la sous-fenetre AutoRadioButton
    ; -------------------------------------------
    stdcall Dlg,Button,AutoRadioButton,[@hWnd],70,130,134,24,ID_AUTORADIOBUTTON,\
                                                          BS_AUTORADIOBUTTON,0h

    ; Cration de la sous-fenetre ThreeState
    ; --------------------------------------
    stdcall Dlg,Button,ThreeState,[@hWnd],70,154,72,24,ID_3STATE,BS_3STATE,0h

    ; Cration de la sous-fenetre Auto3State
    ; --------------------------------------
    stdcall Dlg,Button,Auto3State,[@hWnd],70,180,101,24,ID_AUTO3STATE,BS_AUTO3STATE,0h

    ret
endp

;===========================================================================================
; Cration d'une fenetre-enfant control class
;-------------------------------------------------------------------------------------------
; @Class    = adresse du nom de control class
; @AddrText = adresse du control text
; @hParent  = handle de la fenetre parent
; @X        = x-coordonne fenetre-enfant
; @Y        = y-coordonne fenetre-enfant
; @Xw       = largeur en pixels fenetre-enfant
; @Yh       = hauteur en pixels fenetre-enfant
; @Id       = identificateur du controle forme ID_XXX
; @Style    = complment window style de WS_CHILD+WS_VISIBLE
; @ExStyle  = window style tendu
;-------------------------------------------------------------------------------------------
; sortie eax = handle de la fenetre-enfant
;-------------------------------------------------------------------------------------------
proc Dlg @Class, @AddrText, @hParent, @X, @Y, @Xw, @Yh, @Id, @Style, @ExStyle

    mov eax,WS_CHILD+WS_VISIBLE
    add eax,[@Style]
    invoke CreateWindowEx,[@ExStyle],[@Class],[@AddrText],eax,[@X],[@Y],[@Xw],[@Yh],\
                          [@hParent],[@Id],[wc.hInstance],NULL
    mov ebx,eax

    ; Valider la fonte standard MS Sans Serif pour le control
    ; -------------------------------------------------------
    invoke SendMessage,ebx,WM_SETFONT,[hFontMSSS],NULL
    mov eax,ebx

    ret
endp

;===========================================================================================
; Dfinitions des datas chaines / variables initialises ou non initialises / structures
;-------------------------------------------------------------------------------------------
section '.data' data readable writeable

  Titre       db ' BUTTON control',0
  ClassName   db ' AsmGges Win32',0
  MsgEnd      db ' AsmGges France @2006  ',0
  GbGges      db 'AsmGges Button',0
  NoText      db '',0

  Button      db 'BUTTON',0

  PushButton      db 'PushButton',0
  CheckBox        db 'CheckBox',0
  AutoCheckBox    db 'AutoCheckBox',0
  RadioButton     db 'RadioButton',0
  AutoRadioButton db 'AutoRadioButton',0
  ThreeState      db '3State',0
  Auto3State      db 'Auto3State',0

  MSSansSerif     db 'MS Sans Serif',0

  align 4

  WinX        dd 150
  WinY        dd 100
  WinW        dd 300
  WinH        dd 244

  hWnd        dd ?
  hFontMSSS   dd ?

  wc          WNDCLASS   0, WindowProc, 0, 0, 0, 0, 0, COLOR_BTNFACE+1, 0, ClassName
  msg         MSG

;===========================================================================================
; Inclusion des fichiers .dll contenant les fonctions Windows utilises
;-------------------------------------------------------------------------------------------
section '.idata' import data readable writeable

  library kernel32,'KERNEL32.DLL',\
            user32,'USER32.DLL',\
             gdi32,'GDI32.DLL'

  ; Apis Windows dans fichiers dlls
  ; -------------------------------
  include 'apia\kernel32.inc'
  include 'apia\user32.inc'
  include 'apia\gdi32.inc'

;===========================================================================================
; Inclusion des icnes / curseurs / bitmap
;-------------------------------------------------------------------------------------------
section '.rsrc' resource data readable

  directory RT_ICON,icons,\
            RT_GROUP_ICON,group_icons

  IDI_FRANCE = 17

  resource icons,\
           1,LANG_NEUTRAL,icon_data

  resource group_icons,\
           IDI_FRANCE,LANG_NEUTRAL,main_icon

  icon main_icon,icon_data,'res\france.ico'

;===========================================================================================
